-- Written by David Rotermund (davrot@neuro.uni-bremen.de)
-- University of Bremen, Institute for theoretical physiscs
-- (Use Atom.IO editor for correct indents)

-- Top module for
-- Targeted device: <Family::IGLOO> <Die::AGL250V2> <Package::100 VQFP>



library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity IGLOO_TOP is
  port (


    GPIO                : in  STD_LOGIC;
    --NOT_BUSY            : in  STD_LOGIC;
	NOT_BIN			    : out STD_LOGIC;

    --MISO                : in  STD_LOGIC;
	NOT_LDAC 	        : out STD_LOGIC;
	NOT_CLR			    : out STD_LOGIC;
	NOT_RESET_DAC	    : out STD_LOGIC;
   
    --NOT_PEC             : in  STD_LOGIC;
	MOSI			    : out STD_LOGIC;
	SCLK			    : out STD_LOGIC;
	CS				    : out STD_LOGIC;

    -- Current source NOT_OD control
    CS_SRCLK            : out STD_LOGIC;
    CS_RCLK             : out STD_LOGIC;
    CS_SER              : out STD_LOGIC_VECTOR(1 downto 0);
    CS_TFLAG            : in  STD_LOGIC;

    -- MASTER FPGA ->
    SF_READY            : out STD_LOGIC;
    SF_DATA             : in  STD_LOGIC_VECTOR(7 downto 0);
    SF_ERROR            : out STD_LOGIC_VECTOR(3 downto 0);
    SF_VALID            : in  STD_LOGIC;
    SF_FULL             : out STD_LOGIC;

    ZestTemperaturControl: in  STD_LOGIC;
    
    DEBUG_DATA           : out  STD_LOGIC_VECTOR(7 downto 0);
    DEBUG_DATAVALID      : out  STD_LOGIC;


    GPIO_OUT            : out STD_LOGIC;


    ---- Switches
    SW_CLR              : out STD_LOGIC_VECTOR(1 downto 0);
    SW_NOT_LE           : out STD_LOGIC_VECTOR(1 downto 0);
    SW_CLK              : out STD_LOGIC_VECTOR(1 downto 0);
    SW_DIN              : out STD_LOGIC_VECTOR(7 downto 0);
    
    Clk                 :  in STD_LOGIC;
    Reset               :  in STD_LOGIC
);
end IGLOO_TOP;

architecture IGLOO_TOP_ARCH of IGLOO_TOP is

    signal Reset_Inverted           : STD_LOGIC := '0';

	constant NOT_BIN_SETTING		: STD_LOGIC := '0';
	constant NOT_CLR_SETTING		: STD_LOGIC := '1';
	constant NOT_OD_SETTING 		: std_logic := '1';

	signal NOT_BIN_INTERNAL		    : STD_LOGIC := '0';
	signal NOT_LDAC_INTERNAL	    : STD_LOGIC := '0';
	signal NOT_CLR_INTERNAL			: STD_LOGIC := '0';
	signal NOT_RESET_DAC_INTERNAL	: STD_LOGIC := '0';
	signal MOSI_INTERNAL			: STD_LOGIC := '0';
	signal SCLK_INTERNAL			: STD_LOGIC := '0';
	signal CS_INTERNAL				: STD_LOGIC := '0';

	signal INPUT_PARSER_DATA            : STD_LOGIC_VECTOR(8 downto 0) := (others => '0');
	signal INPUT_PARSER_VALID 			: STD_LOGIC := '0';	                
	signal INPUT_PARSER_FULL			: STD_LOGIC := '0';		            

    -- NEW NETWORK
    signal SF_READY_INTERNAL            : STD_LOGIC := '0';		
    signal SF_READY_COUNTER             : unsigned(7 downto 0) := (others => '0');   

    signal SF_FULL_INTERNAL             : STD_LOGIC := '0';		

    signal Switch_CS					: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
    signal Switch_A						: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Switch_B						: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Switch_C						: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
	signal Switch_D						: STD_LOGIC_VECTOR(15 downto 0) := (others => '0');
    signal UpdateSwitch                 : STD_LOGIC := '0';

    signal INPUT_PARSER_DEBUGDATA       : STD_LOGIC_VECTOR(8 downto 0) := (others => '0');
    signal SF_DATA_Buffer               : STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
    signal SF_VALID_Buffer              : STD_LOGIC := '0';

 begin

    Reset_Inverted <= NOT Reset;

    -- DAC
    NOT_BIN_INTERNAL <= NOT_BIN_SETTING;
    NOT_CLR_INTERNAL <= NOT_CLR_SETTING;


	NOT_BIN <= NOT_BIN_INTERNAL;
	NOT_LDAC <= NOT_LDAC_INTERNAL;
	NOT_CLR <= NOT_CLR_INTERNAL;
	NOT_RESET_DAC <= NOT_RESET_DAC_INTERNAL;
	MOSI <= MOSI_INTERNAL;
	SCLK <= SCLK_INTERNAL;
	CS	<= CS_INTERNAL;

    CopyDebugs : process(Reset_Inverted,CLK)
    begin
        if (Reset_Inverted = '1') then
            GPIO_OUT  <= '0';
            SF_ERROR(3) <= '1';
            DEBUG_DATA <= (others => '0');
            DEBUG_DATAVALID <= '0';

            SF_DATA_Buffer <= (others => '0');
            SF_VALID_Buffer <= '0';

        elsif (rising_edge(CLK)) then
            GPIO_OUT  <= GPIO;
            SF_ERROR(3) <= CS_TFLAG;

            -- SF_DATA_Buffer <= SF_DATA;
            -- SF_VALID_Buffer <= SF_VALID;
            DEBUG_DATA <= SF_DATA;
            DEBUG_DATAVALID <= SF_VALID;

            -- DEBUG_DATA <= SF_DATA_BUFFER;
            -- DEBUG_DATAVALID <= SF_VALID_BUFFER;


            -- DEBUG_DATA <= INPUT_PARSER_DEBUGDATA(7 downto 0);
            -- DEBUG_DATAVALID <= INPUT_PARSER_DEBUGDATA(8);
        end if;
    end process CopyDebugs;

    MyDAC: entity work.DACSwitchCore 
    Port MAP( 
			-- Controll data from an external data source 
			INPUT_FULL   						     => INPUT_PARSER_FULL,
			INPUT_DATA         				         => INPUT_PARSER_DATA,
			INPUT_DATA_VALID    				     => INPUT_PARSER_VALID,
			INPUT_DATA_CLK    				         => Clk,

			-- SPI and Co
			NOT_LDAC		    					=> NOT_LDAC_INTERNAL,
			NOT_RESET_DAC	    				    => NOT_RESET_DAC_INTERNAL, 
			MOSI									=> MOSI_INTERNAL,
			CS										=> CS_INTERNAL,
			SCLK									=> SCLK_INTERNAL,
			
            Switch_CS							    => Switch_CS,
			Switch_A								=> Switch_A,
			Switch_B								=> Switch_B,
			Switch_C								=> Switch_C,
			Switch_D								=> Switch_D,
            
            UpdateSwitch                            => UpdateSwitch,
			
			Reset 								    => Reset_Inverted,
			Clk 									=> Clk
		);

    SF_READY <= SF_READY_INTERNAL;
    SignalReadyness : process (Reset_Inverted, Clk)
    begin
        if Reset_Inverted = '1' then
            SF_READY_INTERNAL <= '1';
            SF_READY_COUNTER <= (others => '0');

        elsif rising_edge(Clk) then
            SF_READY_INTERNAL <= SF_READY_INTERNAL;
            SF_READY_COUNTER <= SF_READY_COUNTER + 1;

            if std_logic_vector(SF_READY_COUNTER) = "11111111" then
                SF_READY_INTERNAL <= NOT SF_READY_INTERNAL;
            else
                null;
            end if; 
    
        end if;
    end process SignalReadyness;

    -- SF_FULL <= NOT SF_FULL_INTERNAL;
    SF_FULL <= INPUT_PARSER_FULL;

    MyInputParser : entity work.InputParser 
    Port MAP ( 

		DATA_IN_EXT 					            => SF_DATA,
		DATA_IN_VALID_EXT 				            => SF_VALID,	
		DATA_IN_READENABLE			                => SF_FULL_INTERNAL,
		
		---- For the strategy it is important that the PROG_FULL of the FIFO gives us some space :-) 
		DATA_OUT_EXT 					            => INPUT_PARSER_DATA,
		DATA_OUT_VALID_EXT			                => INPUT_PARSER_VALID,
		DATA_OUT_FULL					            => INPUT_PARSER_FULL,
		
		ENTRAINMENT_FAILED			                => SF_ERROR(0),
		PACKET_BROKEN					            => SF_ERROR(1),
		
		--  0000000 = Wildcard 
		MYADDRESS						            => "0000001",
		
        DEBUG_DATA                                  => INPUT_PARSER_DEBUGDATA,

        Clk                                         => Clk,
        Reset                                       => Reset_Inverted
	);

    
    MySwitchControl : entity work.SwitchControl
    port MAP(

		Switch_CS								=> Switch_CS,
		Switch_A								=> Switch_A,
		Switch_B								=> Switch_B,
		Switch_C								=> Switch_C,
		Switch_D								=> Switch_D,

        UpdateSwitch                            => UpdateSwitch,

        CS_SRCLK                                => CS_SRCLK,
        CS_RCLK                                 => CS_RCLK,
        CS_SER                                  => CS_SER,
        CS_TFLAG                                => ZestTemperaturControl,
        
        SW_CLR                                  => SW_CLR,
        SW_NOT_LE                               => SW_NOT_LE,
        SW_CLK                                  => SW_CLK,
        SW_DIN                                  => SW_DIN,

        TEMP_PROBLEM                            => SF_ERROR(2),

		Clk                                     => Clk,
		Reset                                   => Reset_Inverted
    );


end architecture IGLOO_TOP_ARCH;

